const colors = {
	greyLight: "#FBFBFB",
  greyMedium: "#BFBFBF",
  blueDark: "#00A0D2",
  yellowDark: "#FFB81C",
  purpleDark: "#702082"
}

let chartDims = {
	width: 400,
  height: 400,
  margin: {
  	top: 30,
    right: 20,
    bottom: 40,
    left: 40
  }
}
chartDims.boundedWidth = chartDims.width 
  - chartDims.margin.left
	- chartDims.margin.right
chartDims.boundedHeight = chartDims.height
	- chartDims.margin.top
 	- chartDims.margin.bottom

const svg = d3.select("svg")
		.attr("width", chartDims.width)
		.attr("height", chartDims.height)

// add a grey background to the chart
svg.append("rect")
		.attr("width", chartDims.width)
		.attr("height", chartDims.height)
		.attr("fill", colors.greyLight)

const bounds = svg.append("g")
		.style("transform", `translate(${chartDims.margin.left}px, 
    	${chartDims.margin.top}px)`)

const moveTransition = d => d3.transition().duration(600)
// const exitTransition = d => moveTransition().transition().duration(400)

const chartText = [{
	label: "13th month persistency(%)",
  x: 238,
  y: 393,
  size: 10
}, {
	label: "61st month persistency(%)",
  x: 15,
  y: 25,
  size: 10
}
] 

const drawChart = async () => {
	const data = await d3.csv("data.csv", d3.autoType)
  
  const xScale = d3.scaleLinear()
		.domain(d3.extent(data, d => d.month_13))
		.range([0, chartDims.boundedWidth])
  	.nice()
  
  const xAxisGenerator = d3.axisBottom()
  	.scale(xScale)
  	.ticks(5)
  
  const yScale = d3.scaleLinear()
  	.domain(d3.extent(data, d => d.month_61))
  	.range([chartDims.boundedHeight, 0])
  	.nice()
  
  const yAxisGenerator = d3.axisLeft()
  	.scale(yScale)
  	.ticks(5)
  
  const positionPoints = selection => selection	
		.attr("cx", d => xScale(d.month_13))
		.attr("cy", d => yScale(d.month_61))
  	.style("fill", colors.blueDark)
		.style("stroke", "white")
		.style("stroke-width", 1)
  
  const growPoints = selection => selection
  	.attr("r", 0)
  	.transition(moveTransition())
  		.attr("r", 6)
  
  const drawPoints = (selection, data) => selection.selectAll("circle")
  	.data(data)
  	.join(
    	enter => enter.append("circle")
      	.call(positionPoints)
      	.call(growPoints),
      update => update
      	.call(update => 
              update.transition(moveTransition())
             		.delay((d, i) => i * 20)
             		.call(positionPoints)),
      exit => exit.remove()
    )
  
  let dataFiltered = data.filter(d => d.year == 2021 && d.month_61 !== "NA")
  
  bounds.append("g").call(xAxisGenerator)
  		.style("transform", `translateY(${chartDims.boundedHeight}px)`)
  	.call(g => g.select(".domain").remove())
  	.call(g => g.selectAll(".tick line")
  		.attr("y1", -chartDims.boundedHeight)
      .style("opacity", 0.15))
   
  bounds.append("g").call(yAxisGenerator)
  	.call(g => g.select(".domain").remove())
  	.call(g => g.selectAll(".tick line")
  		.attr("x1", chartDims.boundedWidth)
      .style("opacity", 0.15))
  
  bounds.call(drawPoints, dataFiltered)
  
  const drag = d3.drag()
  	.on("drag", dragged)
  	.on("end", dragEnded)
  
  function dragged(event, d) {
  	d3.select(this)
      	.attr("x", d.x = event.x)	
    		.attr("y", d.y = event.y)
  }
  
  function dragEnded(event, d) {
  	console.log(d)
  }
  
  const textData = svg.append("g").selectAll("text")
  	.data(chartText)
  	.join("text")
  		.attr("class", "chart-text")
  		.attr("x", d => d.x)
  		.attr("y", d => d.y)
  		.style("font-size", d => d.size)
  		.text(d => d.label)
  		// .call(drag)
  
  const selectYear = d3.select("#year-select")
  selectYear.on("change", (event) => {
  	const currYear = event.target.value
    dataFiltered = data.filter(d => d.year == currYear && d.month_61 !== "NA")
    bounds.call(drawPoints, dataFiltered)
  })
  
}
drawChart()